@@ -4,7 +4,7 @@
  * Date  : 2004-07-20
  * Description : image histogram adjust levels.
  *
- * Copyright 2004 by Gilles Caulier
+ * Copyright 2004-2005 by Gilles Caulier
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -41,6 +41,9 @@
 #include <qlayout.h>
 #include <qframe.h>
 #include <qtimer.h>
+#include <qhbuttongroup.h> 
+#include <qpixmap.h>
+#include <qcheckbox.h>
 
 // KDE includes.
 
@@ -89,7 +92,7 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
                                        digikamimageplugins_version,
                                        I18N_NOOP("An image-histogram-levels adjustment plugin for digiKam."),
                                        KAboutData::License_GPL,
-                                       "(c) 2004, Gilles Caulier",
+                                       "(c) 2004-2005, Gilles Caulier",
                                        0,
                                        "http://extragear.kde.org/apps/digikamimageplugins");
 
@@ -133,7 +136,7 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
 
     QGroupBox *gbox = new QGroupBox(plainPage());
     gbox->setFlat(false);
-    gbox->setTitle(i18n("Input/Output Levels Settings"));
+    gbox->setTitle(i18n("Settings"));
     QGridLayout* grid = new QGridLayout( gbox, 6, 5, 20, spacingHint());
 
     QLabel *label1 = new QLabel(i18n("Channel:"), gbox);
@@ -174,12 +177,7 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
     frame->setFrameStyle(QFrame::Panel|QFrame::Sunken);
     QVBoxLayout* l = new QVBoxLayout(frame, 5, 0);
 
-    m_histogramWidget = new Digikam::HistogramWidget(256, 140,
-                                                     imageData,
-                                                     width,
-                                                     height,
-                                                     frame,
-                                                     false);
+    m_histogramWidget = new Digikam::HistogramWidget(256, 140, imageData, width, height, frame, false);
     QWhatsThis::add( m_histogramWidget, i18n("<p>This is the histogram drawing of the selected image channel"));
     l->addWidget(m_histogramWidget, 0);
     grid->addMultiCellWidget(frame, 1, 1, 0, 4);
@@ -256,37 +254,74 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
     QHGroupBox *gbox3 = new QHGroupBox(i18n("All Channels' Levels"), plainPage());
     m_loadButton = new QPushButton(i18n("&Load..."), gbox3);
     QWhatsThis::add( m_loadButton, i18n("<p>Load levels settings from a Gimp levels text file."));
-    m_saveButton = new QPushButton(i18n("&Save..."), gbox3);
+    m_saveButton = new QPushButton(i18n("&Save As..."), gbox3);
     QWhatsThis::add( m_saveButton, i18n("<p>Save levels settings to a Gimp levels text file."));
     m_autoButton = new QPushButton(i18n("&Auto"), gbox3);
     QWhatsThis::add( m_autoButton, i18n("<p>Adjust levels automatically."));
-    m_resetButton = new QPushButton(i18n("&Reset All Values"), gbox3);
+    m_resetButton = new QPushButton(i18n("&Reset All"), gbox3);
     QWhatsThis::add( m_resetButton, i18n("<p>Reset all channels' level values."));
 
+    m_pickerColorButtonGroup = new QHButtonGroup(gbox3);
+    m_pickBlack = new QPushButton(m_pickerColorButtonGroup);
+    m_pickerColorButtonGroup->insert(m_pickBlack, BlackTonal);
+    KGlobal::dirs()->addResourceType("color-picker-black", KGlobal::dirs()->kde_default("data") +
+                                     "digikamimageplugins/data");
+    directory = KGlobal::dirs()->findResourceDir("color-picker-black", "color-picker-black.png");
+    m_pickBlack->setPixmap( QPixmap( directory + "color-picker-black.png" ) );
+    m_pickBlack->setToggleButton(true);
+    QToolTip::add( m_pickBlack, i18n( "All Channels shadow tone color picker" ) );
+    QWhatsThis::add( m_pickBlack, i18n("<p>With this button, you can pick the color from original image used to set <b>Shadow Tone</b> "
+                                       "levels input on Red, Green, Blue, and Luminosity channels."));
+    m_pickGray  = new QPushButton(m_pickerColorButtonGroup);
+    m_pickerColorButtonGroup->insert(m_pickGray, GrayTonal);
+    KGlobal::dirs()->addResourceType("color-picker-gray", KGlobal::dirs()->kde_default("data") +
+                                     "digikamimageplugins/data");
+    directory = KGlobal::dirs()->findResourceDir("color-picker-gray", "color-picker-gray.png");
+    m_pickGray->setPixmap( QPixmap( directory + "color-picker-gray.png" ) );
+    m_pickGray->setToggleButton(true);
+    QToolTip::add( m_pickGray, i18n( "All Channels middle tone color picker" ) );
+    QWhatsThis::add( m_pickGray, i18n("<p>With this button, you can pick the color from original image used to set <b>Middle Tone</b> "
+                                      "levels input on Red, Green, Blue, and Luminosity channels."));
+    m_pickWhite = new QPushButton(m_pickerColorButtonGroup);
+    m_pickerColorButtonGroup->insert(m_pickWhite, WhiteTonal);
+    KGlobal::dirs()->addResourceType("color-picker-white", KGlobal::dirs()->kde_default("data") +
+                                     "digikamimageplugins/data");
+    directory = KGlobal::dirs()->findResourceDir("color-picker-white", "color-picker-white.png");
+    m_pickWhite->setPixmap( QPixmap( directory + "color-picker-white.png" ) );
+    m_pickWhite->setToggleButton(true);
+    QToolTip::add( m_pickWhite, i18n( "All Channels highlight tone color picker" ) );
+    QWhatsThis::add( m_pickWhite, i18n("<p>With this button, you can pick the color from original image used to set <b>Highlight Tone</b> "
+                                       "levels input on Red, Green, Blue, and Luminosity channels."));
+    m_pickerColorButtonGroup->setExclusive(true);
+    m_pickerColorButtonGroup->setFrameShape(QFrame::NoFrame);    
+    
     topLayout->addMultiCellWidget(gbox3, 3, 3, 0, 0);
 
     // -------------------------------------------------------------
 
-    QVGroupBox *gbox4 = new QVGroupBox(i18n("Image Preview"), plainPage());
+    QVGroupBox *gbox4 = new QVGroupBox(i18n("Preview"), plainPage());
 
-    QLabel *label3 = new QLabel(i18n("Original:"), gbox4);
-    label3->setAlignment ( Qt::AlignHCenter | Qt::AlignVCenter );
     QFrame *frame2 = new QFrame(gbox4);
     frame2->setFrameStyle(QFrame::Panel|QFrame::Sunken);
     QVBoxLayout* l2  = new QVBoxLayout(frame2, 5, 0);
-    m_previewOriginalWidget = new Digikam::ImageWidget(240, 160, frame2);
-    QWhatsThis::add( m_previewOriginalWidget, i18n("<p>You can see here the original image."));
+    m_previewOriginalWidget = new Digikam::ImageGuideWidget(300, 200, frame2, true, 
+                                                            Digikam::ImageGuideWidget::PickColorMode);
+    QWhatsThis::add( m_previewOriginalWidget, i18n("<p>You can see here the original image. You can pick a color on the image using the color "
+                                                   "picker tools to select shadow, middle, and highlight tones to adjust the level points in the Red, "
+                                                   "Green, Blue, and Luminosity Channels."));
     l2->addWidget(m_previewOriginalWidget, 0, Qt::AlignCenter);
 
-    QLabel *label4 = new QLabel(i18n("Target:"), gbox4);
-    label4->setAlignment ( Qt::AlignHCenter | Qt::AlignVCenter );
     QFrame *frame3 = new QFrame(gbox4);
     frame3->setFrameStyle(QFrame::Panel|QFrame::Sunken);
     QVBoxLayout* l3  = new QVBoxLayout(frame3, 5, 0);
-    m_previewTargetWidget = new Digikam::ImageWidget(240, 160, frame3);
+    m_previewTargetWidget = new Digikam::ImageWidget(300, 200, frame3);
     QWhatsThis::add( m_previewTargetWidget, i18n("<p>You can see here the image's level-adjustments preview."));
     l3->addWidget(m_previewTargetWidget, 0, Qt::AlignCenter);
-
+    
+    m_overExposureIndicatorBox = new QCheckBox(i18n("Over exposure indicator"), gbox4);
+    QWhatsThis::add( m_overExposureIndicatorBox, i18n("<p>If you enable this option, over-exposed pixels from the target image preview "
+                                                      "will be over-colored. This will not have an effect on the final rendering."));
+                                                      
     topLayout->addMultiCellWidget(gbox4, 1, 3, 1, 1);
 
     adjustSize();
@@ -303,7 +338,13 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
 
     connect(m_scaleCB, SIGNAL(activated(int)),
             this, SLOT(slotScaleChanged(int)));
+            
+    connect(m_previewOriginalWidget, SIGNAL(spotPositionChanged(  const QColor &, bool, const QPoint & )),
+            this, SLOT(slotSpotColorChanged( const QColor &, bool )));             
 
+    connect(m_overExposureIndicatorBox, SIGNAL(toggled (bool)),
+            this, SLOT(slotEffect()));      
+                        
     // -------------------------------------------------------------
     // Color sliders and spinbox slots.
 
@@ -356,8 +397,7 @@ AdjustLevelDialog::~AdjustLevelDialog()
 
 void AdjustLevelDialog::slotHelp()
 {
-    KApplication::kApplication()->invokeHelp("adjustlevels",
-                                             "digikamimageplugins");
+    KApplication::kApplication()->invokeHelp("adjustlevels", "digikamimageplugins");
 }
 
 void AdjustLevelDialog::closeEvent(QCloseEvent *e)
@@ -367,6 +407,35 @@ void AdjustLevelDialog::closeEvent(QCloseEvent *e)
     e->accept();
 }
 
+void AdjustLevelDialog::slotSpotColorChanged(const QColor &color, bool release)
+{
+    if ( m_pickBlack->isOn() )
+       {
+       // Black tonal levels point.
+       m_levels->levelsBlackToneAdjustByColors(m_channelCB->currentItem(), color);      
+       m_pickBlack->setOn(!release);
+       }
+    else if ( m_pickGray->isOn() )
+       {
+       // Gray tonal levels point.
+       m_levels->levelsGrayToneAdjustByColors(m_channelCB->currentItem(), color);      
+       m_pickGray->setOn(!release);
+       }
+    else if ( m_pickWhite->isOn() )
+       {
+       // White tonal levels point.
+       m_levels->levelsWhiteToneAdjustByColors(m_channelCB->currentItem(), color);      
+       m_pickWhite->setOn(!release);
+       }
+    else
+       m_histogramWidget->setHistogramGuide(color);
+
+    // Refresh the current levels config.
+    slotChannelChanged(m_channelCB->currentItem());
+       
+    slotEffect();                
+}
+
 void AdjustLevelDialog::slotGammaInputchanged(double val)
 {
     blockSignals(true);
@@ -384,8 +453,7 @@ void AdjustLevelDialog::slotAdjustMinInputSpinBox(int val)
 
     m_minInput->setValue(255 - val);
     m_hGradientMinInput->setValue( val );
-    m_levels->setLevelLowInputValue(m_channelCB->currentItem(),
-                                    255 - val);
+    m_levels->setLevelLowInputValue(m_channelCB->currentItem(), 255 - val);
     blockSignals(false);
     slotEffect();
 }
@@ -399,8 +467,7 @@ void AdjustLevelDialog::slotAdjustMaxInputSpinBox(int val)
 
     m_maxInput->setValue(255 - val);
     m_hGradientMaxInput->setValue( val );
-    m_levels->setLevelHighInputValue(m_channelCB->currentItem(),
-                                    255 - val);
+    m_levels->setLevelHighInputValue(m_channelCB->currentItem(), 255 - val);
     blockSignals(false);
     slotEffect();
 }
@@ -414,8 +481,7 @@ void AdjustLevelDialog::slotAdjustMinOutputSpinBox(int val)
 
     m_minOutput->setValue(255 - val);
     m_hGradientMinOutput->setValue( val );
-    m_levels->setLevelLowOutputValue(m_channelCB->currentItem(),
-                                     255 - val);
+    m_levels->setLevelLowOutputValue(m_channelCB->currentItem(), 255 - val);
     blockSignals(false);
     slotEffect();
 }
@@ -429,8 +495,7 @@ void AdjustLevelDialog::slotAdjustMaxOutputSpinBox(int val)
 
     m_maxOutput->setValue(255 - val);
     m_hGradientMaxOutput->setValue( val );
-    m_levels->setLevelHighOutputValue(m_channelCB->currentItem(),
-                                      255 - val);
+    m_levels->setLevelHighOutputValue(m_channelCB->currentItem(), 255 - val);
     blockSignals(false);
     slotEffect();
 }
@@ -458,7 +523,8 @@ void AdjustLevelDialog::slotResetAllChannels()
 
     // Refresh the current levels config.
     slotChannelChanged(m_channelCB->currentItem());
-
+    m_histogramWidget->reset();
+    
     slotEffect();
 }
 
@@ -468,6 +534,7 @@ void AdjustLevelDialog::slotUser1()
 
     // Refresh the current levels config.
     slotChannelChanged(m_channelCB->currentItem());
+    m_histogramWidget->reset();
 
     slotEffect();
 }
@@ -485,11 +552,9 @@ void AdjustLevelDialog::slotAutoLevels()
 
 void AdjustLevelDialog::slotEffect()
 {
-    Digikam::ImageIface* ifaceOrg =
-        m_previewOriginalWidget->imageIface();
+    Digikam::ImageIface* ifaceOrg = m_previewOriginalWidget->imageIface();
 
-    Digikam::ImageIface* ifaceDest =
-        m_previewTargetWidget->imageIface();
+    Digikam::ImageIface* ifaceDest = m_previewTargetWidget->imageIface();
 
     uint* orgData = ifaceOrg->getPreviewData();
     int   w       = ifaceOrg->previewWidth();
@@ -499,7 +564,7 @@ void AdjustLevelDialog::slotEffect()
     uint* desData = new uint[w*h];
 
     // Calculate the LUT to apply on the image.
-    m_levels->levelsLutSetup(Digikam::ImageHistogram::AlphaChannel);
+    m_levels->levelsLutSetup(Digikam::ImageHistogram::AlphaChannel, m_overExposureIndicatorBox->isChecked());
 
     // Apply the lut to the image.
     m_levels->levelsLutProcess(orgData, desData, w, h);
@@ -513,11 +578,9 @@ void AdjustLevelDialog::slotEffect()
 
 void AdjustLevelDialog::slotOk()
 {
-    Digikam::ImageIface* ifaceOrg =
-        m_previewOriginalWidget->imageIface();
+    Digikam::ImageIface* ifaceOrg = m_previewOriginalWidget->imageIface();
 
-    Digikam::ImageIface* ifaceDest =
-        m_previewTargetWidget->imageIface();
+    Digikam::ImageIface* ifaceDest = m_previewTargetWidget->imageIface();
 
     uint* orgData = ifaceOrg->getOriginalData();
     int   w       = ifaceOrg->originalWidth();
@@ -532,7 +595,7 @@ void AdjustLevelDialog::slotOk()
     // Apply the lut to the image.
     m_levels->levelsLutProcess(orgData, desData, w, h);
 
-    ifaceDest->putOriginalData(desData);
+    ifaceDest->putOriginalData(i18n("Adjust Level"), desData);
 
     delete [] orgData;
     delete [] desData;
@@ -543,7 +606,15 @@ void AdjustLevelDialog::slotChannelChanged(int channel)
 {
     switch(channel)
        {
-       case 1:           // Red.
+       case LuminosityChannel:
+          m_histogramWidget->m_channelType = Digikam::HistogramWidget::ValueHistogram;
+          m_hGradientMinInput->setColors( QColor( "black" ), QColor( "white" ) );
+          m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "white" ) );
+          m_hGradientMinOutput->setColors( QColor( "black" ), QColor( "white" ) );
+          m_hGradientMaxOutput->setColors( QColor( "black" ), QColor( "white" ) );
+          break;
+       
+       case RedChannel:
           m_histogramWidget->m_channelType = Digikam::HistogramWidget::RedChannelHistogram;
           m_hGradientMinInput->setColors( QColor( "black" ), QColor( "red" ) );
           m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "red" ) );
@@ -551,7 +622,7 @@ void AdjustLevelDialog::slotChannelChanged(int channel)
           m_hGradientMaxOutput->setColors( QColor( "black" ), QColor( "red" ) );
           break;
 
-       case 2:           // Green.
+       case GreenChannel:
           m_histogramWidget->m_channelType = Digikam::HistogramWidget::GreenChannelHistogram;
           m_hGradientMinInput->setColors( QColor( "black" ), QColor( "green" ) );
           m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "green" ) );
@@ -559,7 +630,7 @@ void AdjustLevelDialog::slotChannelChanged(int channel)
           m_hGradientMaxOutput->setColors( QColor( "black" ), QColor( "green" ) );
           break;
 
-       case 3:           // Blue.
+       case BlueChannel:
           m_histogramWidget->m_channelType = Digikam::HistogramWidget::BlueChannelHistogram;
           m_hGradientMinInput->setColors( QColor( "black" ), QColor( "blue" ) );
           m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "blue" ) );
@@ -567,21 +638,13 @@ void AdjustLevelDialog::slotChannelChanged(int channel)
           m_hGradientMaxOutput->setColors( QColor( "black" ), QColor( "blue" ) );
           break;
 
-       case 4:           // Alpha.
+       case AlphaChannel:
           m_histogramWidget->m_channelType = Digikam::HistogramWidget::AlphaChannelHistogram;
           m_hGradientMinInput->setColors( QColor( "black" ), QColor( "white" ) );
           m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "white" ) );
           m_hGradientMinOutput->setColors( QColor( "black" ), QColor( "white" ) );
           m_hGradientMaxOutput->setColors( QColor( "black" ), QColor( "white" ) );
           break;
-
-       default:          // Luminosity.
-          m_histogramWidget->m_channelType = Digikam::HistogramWidget::ValueHistogram;
-          m_hGradientMinInput->setColors( QColor( "black" ), QColor( "white" ) );
-          m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "white" ) );
-          m_hGradientMinOutput->setColors( QColor( "black" ), QColor( "white" ) );
-          m_hGradientMaxOutput->setColors( QColor( "black" ), QColor( "white" ) );
-          break;
        }
 
     adjustSliders(m_levels->getLevelLowInputValue(channel),
@@ -597,13 +660,13 @@ void AdjustLevelDialog::slotScaleChanged(int scale)
 {
     switch(scale)
        {
-       case 1:           // Log.
-          m_histogramWidget->m_scaleType = Digikam::HistogramWidget::LogScaleHistogram;
-          break;
-
-       default:          // Lin.
+       case Linear: 
           m_histogramWidget->m_scaleType = Digikam::HistogramWidget::LinScaleHistogram;
           break;
+       
+       case Logarithmic:
+          m_histogramWidget->m_scaleType = Digikam::HistogramWidget::LogScaleHistogram;
+          break;
        }
 
     m_histogramWidget->repaint(false);
@@ -633,7 +696,7 @@ void AdjustLevelDialog::slotSaveLevels()
 {
     KURL saveLevelsFile;
 
-    saveLevelsFile = KFileDialog::getOpenURL(KGlobalSettings::documentPath(),
+    saveLevelsFile = KFileDialog::getSaveURL(KGlobalSettings::documentPath(),
                                              QString( "*" ), this,
                                              QString( i18n("Gimp Levels File to Save")) );
     if( saveLevelsFile.isEmpty() )
@@ -649,7 +712,6 @@ void AdjustLevelDialog::slotSaveLevels()
     slotChannelChanged(m_channelCB->currentItem());
 }
 
-
 }  // NameSpace DigikamAdjustLevelsImagesPlugin
 
 #include "adjustlevels.moc"
